#include <iostream>
//#include <stdio.h>
#include "libusb.h"
//#include <libusb-1.0/libusb.h>
#include <unistd.h>
#include <stdlib.h>

/*
Dev (bus 3, device 3): USBWatchDog - USBWatchDog
  Configuration:
    wTotalLength:         34
    bNumInterfaces:       1
    bConfigurationValue:  1
    iConfiguration:       0
    bmAttributes:         80h
    MaxPower:             25
    Interface:
      bInterfaceNumber:   0
      bAlternateSetting:  0
      bNumEndpoints:      1
      bInterfaceClass:    3
      bInterfaceSubClass: 0
      bInterfaceProtocol: 0
      iInterface:         0
      Endpoint:
        bEndpointAddress: 81h
        bmAttributes:     03h
        wMaxPacketSize:   8
        bInterval:        100
        bRefresh:         0
        bSynchAddress:    0

в нём, как написано на сайте, нужно установить фильтр для моего устройства по
VID: 0x16c0 и PID: 0x03e8

CurrentUSBDev:= TUSBDeviceItem.Create(VID, PID);
  try
    CurrentUSBDev.SendComand(0, 0, (ResetAfter * 60));
 Вот отправка команды тестовой перезагрузки:

  CurrentUSBDev:= TUSBDeviceItem.Create(VID, PID);
  try
    CurrentUSBDev.SendComand(0, 1, (ResetAfter * 60));
 первый параметр статичный 0
второй тип операции 0 или 1
третий - это время перезагрузки в секундах. ResetAfter - это параметр из ComboBox-а от 1 до 10ти



 */

using namespace std;

const int VID = 0x16c0;
const int PID = 0x03e8;

/**
 * USBRQ_IO_WRITE: integer = $12; // в usbdrv.h отсутствует! Нужно добавлять ручками. указано как 0x12 в десятично 18
  (* Request-ы  USBRQ_IO_READ и USBRQ_IO_WRITE здесь используются для отправки не пакетированных данных на устройстви
    с использованием функции usb_control_msg. В прошивке МК, в методе usbFunctionSetup обрабатываются по:
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_VENDOR) *)
 */
const int USBRQ_IO_WRITE = 0x12;

/**
 * Sleep seconds in loop
 */
int LOOP_SLEEP = 5;

/**
 * Don't use loop, single run
 */
bool NO_LOOP = false;

uint16_t TIMEOUT = 60*3; //default timeout
uint16_t TEST = 0;

int main(int argc, char *argv[]) {
    if (argc < 2) {
        cout << "Usage examples:" << endl;
        cout << "watchdoginua TIMEOUT - set timeout seconds in endless loop every 5 seconds" << endl;
        cout << "watchdoginua TIMEOUT single - set timeout and exit, no loop, useful for cron" << endl;
        cout << "watchdoginua TIMEOUT testreset - set timeout and exit, watchdog fires instantly" << endl;
        cout << "watchdoginua TIMEOUT testpower - same as testreset but for power pins" << endl;
        cout << "Default TIMEOUT is 180 seconds" << endl;
        return 1;
    }

    TIMEOUT = atoi(argv[1]);
    if (!TIMEOUT) {
        cout << "Invalid timeout: " << argv[1] << endl;
        return 1;
    }

    if (argc>=3) {
        std::string arg2 = argv[2];
        if (arg2 == "single") {
            cout << "Single run" << endl;
            NO_LOOP = true;
        } else if (arg2 == "testreset") {
            cout << "Testing RESET mode" << endl;
            TEST = 1;
            NO_LOOP = true;
        } else if (arg2 == "testpower") {
            cout << "Testing POWER OFF mode" << endl;
            TEST = 2;
            NO_LOOP = true;
        } else {
            cout << "Unknown parameter: \"" << argv[2] << "\"" << endl;
            return 1;
        }
    }


    libusb_device **devs; //pointer to pointer of device, used to retrieve a list of devices
    libusb_device_handle *dev_handle; //a device handle
    libusb_context *ctx = NULL; //a libusb session
    int r; //for return values
    ssize_t cnt; //holding number of devices in list


    r = libusb_init(&ctx); //initialize the library for the session we just declared
    if (r < 0) {
        cout << "init() error " << r << endl; //there was an error
        return 1;
    }
    libusb_set_debug(ctx, 3); //set verbosity level to 3, as suggested in the documentation


    cnt = libusb_get_device_list(ctx, &devs); //get the list of devices
    if (cnt < 0) {
        cout << "get_device_list() error" << endl; //there was an error
        return 1;
    }
    //cout << cnt << " USB Devices in list" << endl;



    dev_handle = libusb_open_device_with_vid_pid(ctx, VID, PID); //these are vendorID and productID I found for my usb device
    if (dev_handle == NULL) {
        cout << "Cannot open device or it's not found" << endl;
        return 1;
    }
    //cout << "Device Opened" << endl;
    libusb_free_device_list(devs, 1); //free the list, unref the devices in it


    if (libusb_kernel_driver_active(dev_handle, 0) == 1) { //find out if kernel driver is attached
        cout << "Kernel Driver Active" << endl;
        if (libusb_detach_kernel_driver(dev_handle, 0) == 0) //detach it
            cout << "Kernel Driver Detached!" << endl;
    }


//    r = libusb_set_configuration(dev_handle, 1);
//    if (r < 0) {
//        cout << "Cannot Set Configuration" << endl;
//        return 1;
//    }
//    cout << "Configuration Set" << endl;

    r = libusb_claim_interface(dev_handle, 0);
    if (r < 0) {
        cout << "Cannot Claim Interface" << endl;
        return 1;
    }
    //cout << "Claimed Interface 0" << endl;


    unsigned int i = 1;
    while (true) {
        r = libusb_control_transfer(dev_handle, LIBUSB_REQUEST_TYPE_VENDOR, USBRQ_IO_WRITE, TEST, TIMEOUT, NULL, 0, 0);
        if (r < 0) {
            cout << "Error during control transfer " << r << std::endl;
            return 1;
        }
        cout << i << ") Sent " << TIMEOUT << " seconds timeout" << endl;

        if (NO_LOOP)
            break;

        sleep(LOOP_SLEEP);
        i++;
    }



    r = libusb_release_interface(dev_handle, 0); //release the claimed interface
    if (r != 0) {
        cout << "Cannot Release Interface" << endl;
        //return 1;
    }
    //cout << "Released Interface" << endl;

    libusb_close(dev_handle); //close the device we opened
    libusb_exit(ctx); //needs to be called to end the

    return 0;
}



//    r = libusb_set_interface_alt_setting(dev_handle, 0, 0x2);
//    if(r != 0) {
//        std::cout << "Cannot configure alternate setting errno=" << r << std::endl;
//        return 3;
//    }



/*
unsigned int DATA_SIZE = 3;
unsigned char *data = new unsigned char[DATA_SIZE+1]; //data to write
data[0] = 0;
data[1] = 1;
data[2] = 5;
data[3] = 0;
//printf(">%s<\n", data);
cout << "Data size = " << DATA_SIZE << endl;
cout << "Data->" << data << "<-" << endl; //just to see the data we want to write : abcd


int actual; //used to find out how many bytes were written
cout << "Writing Data..." << endl;
//r = libusb_bulk_transfer(dev_handle, (2 | LIBUSB_ENDPOINT_OUT), data, DATA_SIZE, &actual, 0);
r = libusb_bulk_transfer(dev_handle, (0x81 | LIBUSB_ENDPOINT_OUT ), data, DATA_SIZE, &actual, 2000);
//my device's out endpoint was 2, found with trial- the device had 2 endpoints: 2 and 129
if (r == 0 && actual == DATA_SIZE) //we wrote the 4 bytes successfully
    cout << "Writing Successful!" << endl;
else
    cout << "Write Error errno=" << r << endl;
delete[] data; //delete the allocated memory for data
*/